@@ -1,8 +1,8 @@
 # Natural Language Toolkit: Corpus & Model Downloader
 #
-# Copyright (C) 2001-2012 NLTK Project
-# Author: Edward Loper <edloper@gradient.cis.upenn.edu>
-# URL: <http://www.nltk.org/>
+# Copyright (C) 2001-2014 NLTK Project
+# Author: Edward Loper <edloper@gmail.com>
+# URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
 
 """
@@ -62,11 +62,13 @@ Usage::
 
     python nltk/downloader.py [-d DATADIR] [-q] [-f] [-k] PACKAGE_IDS
 
-or with py2.5+:
+or::
 
     python -m nltk.downloader [-d DATADIR] [-q] [-f] [-k] PACKAGE_IDS
 """
 #----------------------------------------------------------------------
+from __future__ import print_function, division, unicode_literals
+
 """
 
   0     1  2    3
@@ -157,18 +159,14 @@ they didn't download that model.
 default: unzip or not?
 
 """
-import time, re, os, zipfile, sys, textwrap, threading, itertools
-from cStringIO import StringIO
-try:
-    from hashlib import md5
-except:
-    from md5 import md5
+import time, os, zipfile, sys, textwrap, threading, itertools
+from hashlib import md5
 
 try:
     TKINTER = True
-    from Tkinter import (Tk, Frame, Label, Entry, Button, Canvas, Menu, IntVar,
+    from tkinter import (Tk, Frame, Label, Entry, Button, Canvas, Menu, IntVar,
                          TclError)
-    from tkMessageBox import showerror
+    from tkinter.messagebox import showerror
     from nltk.draw.table import Table
     from nltk.draw.util import ShowText
 except:
@@ -177,13 +175,15 @@ except:
 
 from xml.etree import ElementTree
 import nltk
-urllib2 = nltk.internals.import_from_stdlib('urllib2')
+from nltk import compat
+#urllib2 = nltk.internals.import_from_stdlib('urllib2')
 
 
 ######################################################################
 # Directory entry objects (from the data server's index file)
 ######################################################################
 
+@compat.python_2_unicode_compatible
 class Package(object):
     """
     A directory entry for a downloadable package.  These entries are
@@ -253,13 +253,19 @@ class Package(object):
 
     @staticmethod
     def fromxml(xml):
-        if isinstance(xml, basestring):
+        if isinstance(xml, compat.string_types):
             xml = ElementTree.parse(xml)
+        for key in xml.attrib:
+            xml.attrib[key] = compat.text_type(xml.attrib[key])
         return Package(**xml.attrib)
 
+    def __lt__(self, other):
+        return self.id < other.id
+
     def __repr__(self):
         return '<Package %s>' % self.id
 
+@compat.python_2_unicode_compatible
 class Collection(object):
     """
     A directory entry for a collection of downloadable packages.
@@ -286,11 +292,16 @@ class Collection(object):
 
     @staticmethod
     def fromxml(xml):
-        if isinstance(xml, basestring):
+        if isinstance(xml, compat.string_types):
             xml = ElementTree.parse(xml)
+        for key in xml.attrib:
+            xml.attrib[key] = compat.text_type(xml.attrib[key])
         children = [child.get('ref') for child in xml.findall('item')]
         return Collection(children=children, **xml.attrib)
 
+    def __lt__(self, other):
+        return self.id < other.id
+
     def __repr__(self):
         return '<Collection %s>' % self.id
 
@@ -436,11 +447,11 @@ class Downloader(object):
         lines = 0 # for more_prompt
         if download_dir is None:
             download_dir = self._download_dir
-            print 'Using default data directory (%s)' % download_dir
+            print('Using default data directory (%s)' % download_dir)
         if header:
-            print '='*(26+len(self._url))
-            print ' Data server index for <%s>' % self._url
-            print '='*(26+len(self._url))
+            print('='*(26+len(self._url)))
+            print(' Data server index for <%s>' % self._url)
+            print('='*(26+len(self._url)))
             lines += 3 # for more_prompt
         stale = partial = False
 
@@ -448,9 +459,9 @@ class Downloader(object):
         if show_packages: categories.append('packages')
         if show_collections: categories.append('collections')
         for category in categories:
-            print '%s:' % category.capitalize()
+            print('%s:' % category.capitalize())
             lines += 1 # for more_prompt
-            for info in sorted(getattr(self, category)()):
+            for info in sorted(getattr(self, category)(), key=str):
                 status = self.status(info, download_dir)
                 if status == self.INSTALLED and skip_installed: continue
                 if status == self.STALE: stale = True
@@ -459,17 +470,17 @@ class Downloader(object):
                           self.PARTIAL:'P', self.NOT_INSTALLED: ' '}[status]
                 name = textwrap.fill('-'*27 + (info.name or info.id),
                                      75, subsequent_indent=27*' ')[27:]
-                print '  [%s] %s %s' % (prefix, info.id.ljust(20, '.'), name)
+                print('  [%s] %s %s' % (prefix, info.id.ljust(20, '.'), name))
                 lines += len(name.split('\n')) # for more_prompt
                 if more_prompt and lines > 20:
-                    user_input = raw_input("Hit Enter to continue: ")
+                    user_input = compat.raw_input("Hit Enter to continue: ")
                     if (user_input.lower() in ('x', 'q')): return
                     lines = 0
-            print
+            print()
         msg = '([*] marks installed packages'
         if stale: msg += '; [-] marks out-of-date or corrupt packages'
         if partial: msg += '; [P] marks partially installed collections'
-        print textwrap.fill(msg+')', subsequent_indent=' ', width=76)
+        print(textwrap.fill(msg+')', subsequent_indent=' ', width=76))
 
     def packages(self):
         self._update_index()
@@ -494,7 +505,7 @@ class Downloader(object):
     #/////////////////////////////////////////////////////////////////
 
     def _info_or_id(self, info_or_id):
-        if isinstance(info_or_id, basestring):
+        if isinstance(info_or_id, compat.string_types):
             return self.info(info_or_id)
         else:
             return info_or_id
@@ -522,7 +533,7 @@ class Downloader(object):
 
         # Look up the requested collection or package.
         try: info = self._info_or_id(info_or_id)
-        except (IOError, ValueError), e:
+        except (IOError, ValueError) as e:
             yield ErrorMessage(None, 'Error loading %s: %s' %
                                (info_or_id, e))
             return
@@ -547,7 +558,7 @@ class Downloader(object):
         # Look up the requested items.
         for i in range(len(items)):
             try: items[i] = self._info_or_id(items[i])
-            except (IOError, ValueError), e:
+            except (IOError, ValueError) as e:
                 yield ErrorMessage(items[i], e)
                 return
 
@@ -555,8 +566,10 @@ class Downloader(object):
         num_packages = sum(self._num_packages(item) for item in items)
         progress = 0
         for i, item in enumerate(items):
-            if isinstance(item, Package): delta = 1./num_packages
-            else: delta = float(len(item.packages))/num_packages
+            if isinstance(item, Package):
+                delta = 1./num_packages
+            else:
+                delta = float(len(item.packages))/num_packages
             for msg in self.incr_download(item, download_dir, force):
                 if isinstance(msg, ProgressMessage):
                     yield ProgressMessage(progress + msg.progress*delta)
@@ -598,19 +611,18 @@ class Downloader(object):
         yield StartDownloadMessage(info)
         yield ProgressMessage(5)
         try:
-            infile = urllib2.urlopen(info.url)
-            outfile = open(filepath, 'wb')
-            #print info.size
-            num_blocks = max(1, float(info.size)/(1024*16))
-            for block in itertools.count():
-                s = infile.read(1024*16) # 16k blocks.
-                outfile.write(s)
-                if not s: break
-                if block % 2 == 0: # how often?
-                    yield ProgressMessage(min(80, 5+75*(block/num_blocks)))
+            infile = compat.urlopen(info.url)
+            with open(filepath, 'wb') as outfile:
+                #print info.size
+                num_blocks = max(1, float(info.size)/(1024*16))
+                for block in itertools.count():
+                    s = infile.read(1024*16) # 16k blocks.
+                    outfile.write(s)
+                    if not s: break
+                    if block % 2 == 0: # how often?
+                        yield ProgressMessage(min(80, 5+75*(block/num_blocks)))
             infile.close()
-            outfile.close()
-        except IOError, e:
+        except IOError as e:
             yield ErrorMessage(info, 'Error downloading %r from <%s>:'
                                '\n  %s' % (info.id, info.url, e))
             return
@@ -647,8 +659,8 @@ class Downloader(object):
         else:
             # Define a helper function for displaying output:
             def show(s, prefix2=''):
-                print textwrap.fill(s, initial_indent=prefix+prefix2,
-                                    subsequent_indent=prefix+prefix2+' '*4)
+                print(textwrap.fill(s, initial_indent=prefix+prefix2,
+                                    subsequent_indent=prefix+prefix2+' '*4))
 
             for msg in self.incr_download(info_or_id, download_dir, force):
                 # Error messages
@@ -660,8 +672,8 @@ class Downloader(object):
                         return False
                     self._errors = True
                     if not quiet:
-                        print "Error installing package. Retry? [n/y/e]"
-                        choice = raw_input().strip()
+                        print("Error installing package. Retry? [n/y/e]")
+                        choice = compat.raw_input().strip()
                         if choice in ['y', 'Y']:
                             if not self.download(msg.package.id, download_dir,
                                                  quiet, force, prefix,
@@ -676,20 +688,20 @@ class Downloader(object):
                     if isinstance(msg, StartCollectionMessage):
                         show('Downloading collection %r' % msg.collection.id)
                         prefix += '   | '
-                        print prefix
+                        print(prefix)
                     elif isinstance(msg, FinishCollectionMessage):
-                        print prefix
+                        print(prefix)
                         prefix = prefix[:-4]
                         if self._errors:
                             show('Downloaded collection %r with errors' %
                                  msg.collection.id)
                         else:
-                            show('Done downloading collection %r' %
+                            show('Done downloading collection %s' %
                                  msg.collection.id)
 
                     # Package downloading messages:
                     elif isinstance(msg, StartPackageMessage):
-                        show('Downloading package %r to %s...' %
+                        show('Downloading package %s to %s...' %
                              (msg.package.id, download_dir))
                     elif isinstance(msg, UpToDateMessage):
                         show('Package %s is already up-to-date!' %
@@ -811,7 +823,7 @@ class Downloader(object):
 
         # Download the index file.
         self._index = nltk.internals.ElementWrapper(
-            ElementTree.parse(urllib2.urlopen(self._url)).getroot())
+            ElementTree.parse(compat.urlopen(self._url)).getroot())
         self._index_timestamp = time.time()
 
         # Build a dictionary of packages.
@@ -878,12 +890,12 @@ class Downloader(object):
     # URL & Data Directory
     #/////////////////////////////////////////////////////////////////
 
-    def _get_url(self): 
+    def _get_url(self):
         """The URL for the data server's index file."""
         return self._url
     def _set_url(self, url):
         """
-        Set a new URL for the data server. If we're unable to contact 
+        Set a new URL for the data server. If we're unable to contact
         the given url, then the original url is kept.
         """
         original_url = self._url
@@ -930,7 +942,7 @@ class Downloader(object):
         # append "nltk_data" to the home directory
         return os.path.join(homedir, 'nltk_data')
 
-    def _get_download_dir(self): 
+    def _get_download_dir(self):
         """
         The default directory to which packages will be downloaded.
         This defaults to the value returned by ``default_download_dir()``.
@@ -964,27 +976,26 @@ class DownloaderShell(object):
         self._ds = dataserver
 
     def _simple_interactive_menu(self, *options):
-        print '-'*75
-        spc = (68 - sum(len(o) for o in options))/(len(options)-1)*' '
-        print '    ' + spc.join(options)
+        print('-'*75)
+        spc = (68 - sum(len(o) for o in options))//(len(options)-1)*' '
+        print('    ' + spc.join(options))
         #w = 76/len(options)
         #fmt = '  ' + ('%-'+str(w)+'s')*(len(options)-1) + '%s'
         #print fmt % options
-        print '-'*75
-
+        print('-'*75)
 
     def run(self):
-        print 'NLTK Downloader'
+        print('NLTK Downloader')
         while True:
             self._simple_interactive_menu(
                 'd) Download', 'l) List', ' u) Update', 'c) Config', 'h) Help', 'q) Quit')
-            user_input = raw_input('Downloader> ').strip()
-            if not user_input: print; continue
+            user_input = compat.raw_input('Downloader> ').strip()
+            if not user_input: print(); continue
             command = user_input.lower().split()[0]
             args = user_input.split()[1:]
             try:
                 if command == 'l':
-                    print
+                    print()
                     self._ds.list(self._ds.download_dir, header=False,
                                   more_prompt=True)
                 elif command == 'h':
@@ -998,25 +1009,25 @@ class DownloaderShell(object):
                 elif command == 'u':
                     self._simple_interactive_update()
                 else:
-                    print 'Command %r unrecognized' % user_input
-            except urllib2.HTTPError, e:
-                print 'Error reading from server: %s'%e
-            except urllib2.URLError, e:
-                print 'Error connecting to server: %s'%e.reason
+                    print('Command %r unrecognized' % user_input)
+            except compat.HTTPError as e:
+                print('Error reading from server: %s'%e)
+            except compat.URLError as e:
+                print('Error connecting to server: %s'%e.reason)
             # try checking if user_input is a package name, &
             # downloading it?
-            print
+            print()
 
     def _simple_interactive_download(self, args):
         if args:
             for arg in args:
                 try: self._ds.download(arg, prefix='    ')
-                except (IOError, ValueError), e: print e
+                except (IOError, ValueError) as e: print(e)
         else:
             while True:
-                print
-                print 'Download which package (l=list; x=cancel)?'
-                user_input = raw_input('  Identifier> ')
+                print()
+                print('Download which package (l=list; x=cancel)?')
+                user_input = compat.raw_input('  Identifier> ')
                 if user_input.lower()=='l':
                     self._ds.list(self._ds.download_dir, header=False,
                                   more_prompt=True, skip_installed=True)
@@ -1026,86 +1037,86 @@ class DownloaderShell(object):
                 elif user_input:
                     for id in user_input.split():
                         try: self._ds.download(id, prefix='    ')
-                        except (IOError, ValueError), e: print e
+                        except (IOError, ValueError) as e: print(e)
                     break
 
     def _simple_interactive_update(self):
         while True:
             stale_packages = []
             stale = partial = False
-            for info in sorted(getattr(self._ds, 'packages')()):
+            for info in sorted(getattr(self._ds, 'packages')(), key=str):
                 if self._ds.status(info) == self._ds.STALE:
                     stale_packages.append((info.id, info.name))
 
-            print
+            print()
             if stale_packages:
-                print 'Will update following packages (o=ok; x=cancel)'
+                print('Will update following packages (o=ok; x=cancel)')
                 for pid, pname in stale_packages:
                     name = textwrap.fill('-'*27 + (pname),
                                      75, subsequent_indent=27*' ')[27:]
-                    print '  [ ] %s %s' % (pid.ljust(20, '.'), name)
-                print
+                    print('  [ ] %s %s' % (pid.ljust(20, '.'), name))
+                print()
 
-                user_input = raw_input('  Identifier> ')
+                user_input = compat.raw_input('  Identifier> ')
                 if user_input.lower()=='o':
                     for pid, pname in stale_packages:
                         try: self._ds.download(pid, prefix='    ')
-                        except (IOError, ValueError), e: print e
+                        except (IOError, ValueError) as e: print(e)
                     break
                 elif user_input.lower() in ('x', 'q', ''):
                     return
             else:
-                print 'Nothing to update.'
+                print('Nothing to update.')
                 return
 
     def _simple_interactive_help(self):
-        print
-        print 'Commands:'
-        print '  d) Download a package or collection     u) Update out of date packages'
-        print '  l) List packages & collections          h) Help'
-        print '  c) View & Modify Configuration          q) Quit'
+        print()
+        print('Commands:')
+        print('  d) Download a package or collection     u) Update out of date packages')
+        print('  l) List packages & collections          h) Help')
+        print('  c) View & Modify Configuration          q) Quit')
 
     def _show_config(self):
-        print
-        print 'Data Server:'
-        print '  - URL: <%s>' % self._ds.url
-        print ('  - %d Package Collections Available' %
-               len(self._ds.collections()))
-        print ('  - %d Individual Packages Available' %
-               len(self._ds.packages()))
-        print
-        print 'Local Machine:'
-        print '  - Data directory: %s' % self._ds.download_dir
+        print()
+        print('Data Server:')
+        print('  - URL: <%s>' % self._ds.url)
+        print(('  - %d Package Collections Available' %
+               len(self._ds.collections())))
+        print(('  - %d Individual Packages Available' %
+               len(self._ds.packages())))
+        print()
+        print('Local Machine:')
+        print('  - Data directory: %s' % self._ds.download_dir)
 
     def _simple_interactive_config(self):
         self._show_config()
         while True:
-            print
+            print()
             self._simple_interactive_menu(
                 's) Show Config', 'u) Set Server URL',
                 'd) Set Data Dir', 'm) Main Menu')
-            user_input = raw_input('Config> ').strip().lower()
+            user_input = compat.raw_input('Config> ').strip().lower()
             if user_input == 's':
                 self._show_config()
             elif user_input == 'd':
-                new_dl_dir = raw_input('  New Directory> ').strip().lower()
+                new_dl_dir = compat.raw_input('  New Directory> ').strip().lower()
                 if new_dl_dir in ('', 'x', 'q'):
-                    print '  Cancelled!'
+                    print('  Cancelled!')
                 elif os.path.isdir(new_dl_dir):
                     self._ds.download_dir = new_dl_dir
                 else:
-                    print ('Directory %r not found!  Create it first.' %
-                           new_dl_dir)
+                    print(('Directory %r not found!  Create it first.' %
+                           new_dl_dir))
             elif user_input == 'u':
-                new_url = raw_input('  New URL> ').strip().lower()
+                new_url = compat.raw_input('  New URL> ').strip().lower()
                 if new_url in ('', 'x', 'q'):
-                    print '  Cancelled!'
+                    print('  Cancelled!')
                 else:
                     if not new_url.startswith('http://'):
                         new_url = 'http://'+new_url
                     try: self._ds.url = new_url
-                    except Exception, e:
-                        print 'Error reading <%r>:\n  %s' % (new_url, e)
+                    except Exception as e:
+                        print('Error reading <%r>:\n  %s' % (new_url, e))
             elif user_input == 'm':
                 break
 
@@ -1122,7 +1133,7 @@ class DownloaderGUI(object):
     COLUMNS = ['', 'Identifier', 'Name', 'Size', 'Status',
                'Unzipped Size',
                'Copyright', 'Contact', 'License', 'Author',
-               'SVN Revision', 'Subdir', 'Checksum']
+               'Subdir', 'Checksum']
     """A list of the names of columns.  This controls the order in
        which the columns will appear.  If this is edited, then
        ``_package_to_columns()`` may need to be edited to match."""
@@ -1215,9 +1226,9 @@ class DownloaderGUI(object):
         self._init_menu()
         try:
             self._fill_table()
-        except urllib2.HTTPError, e:
+        except compat.HTTPError as e:
             showerror('Error reading from server', e)
-        except urllib2.URLError, e:
+        except compat.URLError as e:
             showerror('Error connecting to server', e.reason)
 
         self._show_info()
@@ -1397,9 +1408,9 @@ class DownloaderGUI(object):
         self._ds.clear_status_cache()
         try:
             self._fill_table()
-        except urllib2.HTTPError, e:
+        except compat.HTTPError as e:
             showerror('Error reading from server', e)
-        except urllib2.URLError, e:
+        except compat.URLError as e:
             showerror('Error connecting to server', e.reason)
         self._table.select(0)
 
@@ -1424,7 +1435,7 @@ class DownloaderGUI(object):
 
     def _table_reprfunc(self, row, col, val):
         if self._table.column_names[col].endswith('Size'):
-            if isinstance(val, basestring): return '  %s' % val
+            if isinstance(val, compat.string_types): return '  %s' % val
             elif val < 1024**2: return '  %.1f KB' % (val/1024.**1)
             elif val < 1024**3: return '  %.1f MB' % (val/1024.**2)
             else: return '  %.1f GB' % (val/1024.**3)
@@ -1437,7 +1448,7 @@ class DownloaderGUI(object):
         try:
             self._ds.url = url
             self._fill_table()
-        except IOError, e:
+        except IOError as e:
             showerror('Error Setting Server Index', str(e))
         self._show_info()
 
@@ -1450,14 +1461,14 @@ class DownloaderGUI(object):
         self._ds.download_dir = download_dir
         try:
             self._fill_table()
-        except urllib2.HTTPError, e:
+        except compat.HTTPError as e:
             showerror('Error reading from server', e)
-        except urllib2.URLError, e:
+        except compat.URLError as e:
             showerror('Error connecting to server', e.reason)
         self._show_info()
 
     def _show_info(self):
-        print 'showing info', self._ds.url
+        print('showing info', self._ds.url)
         for entry,cb in self._info.values():
             entry['state'] = 'normal'
             entry.delete(0, 'end')
@@ -1472,9 +1483,9 @@ class DownloaderGUI(object):
                 self._tab = self._tab_names[i-1].lower()
                 try:
                     return self._fill_table()
-                except urllib2.HTTPError, e:
+                except compat.HTTPError as e:
                     showerror('Error reading from server', e)
-                except urllib2.URLError, e:
+                except compat.URLError as e:
                     showerror('Error connecting to server', e.reason)
 
     def _next_tab(self, *e):
@@ -1483,18 +1494,18 @@ class DownloaderGUI(object):
                 self._tab = self._tab_names[i+1].lower()
                 try:
                     return self._fill_table()
-                except urllib2.HTTPError, e:
+                except compat.HTTPError as e:
                     showerror('Error reading from server', e)
-                except urllib2.URLError, e:
+                except compat.URLError as e:
                     showerror('Error connecting to server', e.reason)
 
     def _select_tab(self, event):
         self._tab = event.widget['text'].lower()
         try:
             self._fill_table()
-        except urllib2.HTTPError, e:
+        except compat.HTTPError as e:
             showerror('Error reading from server', e)
-        except urllib2.URLError, e:
+        except compat.URLError as e:
             showerror('Error connecting to server', e.reason)
 
     _tab = 'collections'
@@ -1564,7 +1575,7 @@ class DownloaderGUI(object):
 
     _DL_DELAY=10
     def _download_cb(self, download_iter, ids):
-        try: msg = download_iter.next()
+        try: msg = next(download_iter)
         except StopIteration:
             #self._fill_table(sort=False)
             self._update_table_status()
@@ -1584,10 +1595,10 @@ class DownloaderGUI(object):
             self._show_progress(None)
             return # halt progress.
         elif isinstance(msg, StartCollectionMessage):
-            show('Downloading collection %r' % msg.collection.id)
+            show('Downloading collection %s' % msg.collection.id)
             self._log_indent += 1
         elif isinstance(msg, StartPackageMessage):
-            show('Downloading package %r' % msg.package.id)
+            show('Downloading package %s' % msg.package.id)
         elif isinstance(msg, UpToDateMessage):
             show('Package %s is up-to-date!' % msg.package.id)
         #elif isinstance(msg, StaleMessage):
@@ -1749,7 +1760,7 @@ class DownloaderGUI(object):
     def _init_progressbar(self):
         c = self._progressbar
         width, height = int(c['width']), int(c['height'])
-        for i in range(0, (int(c['width'])*2)/self._gradient_width):
+        for i in range(0, (int(c['width'])*2)//self._gradient_width):
             c.create_line(i*self._gradient_width+20, -20,
                           i*self._gradient_width-height-20, height+20,
                           width=self._gradient_width,
@@ -1768,7 +1779,7 @@ class DownloaderGUI(object):
             c.itemconfig('gradient', state='hidden')
         else:
             width, height = int(c['width']), int(c['height'])
-            x = percent * int(width) / 100 + 1
+            x = percent * int(width) // 100 + 1
             c.coords('redbox', 0, 0, x, height+1)
 
     def _progress_alive(self):
@@ -1944,16 +1955,20 @@ def md5_hexdigest(file):
     Calculate and return the MD5 checksum for a given file.
     ``file`` may either be a filename or an open stream.
     """
-    if isinstance(file, basestring):
-        file = open(file, 'rb')
+    if isinstance(file, compat.string_types):
+        with open(file, 'rb') as infile:
+            return _md5_hexdigest(infile)
+    return _md5_hexdigest(file)
 
+def _md5_hexdigest(fp):
     md5_digest = md5()
     while True:
-        block = file.read(1024*16) # 16k blocks
+        block = fp.read(1024*16)  # 16k blocks
         if not block: break
         md5_digest.update(block)
     return md5_digest.hexdigest()
 
+
 # change this to periodically yield progress messages?
 # [xx] get rid of topdir parameter -- we should be checking
 # this when we build the index, anyway.
@@ -1964,7 +1979,7 @@ def unzip(filename, root, verbose=True):
     """
     for message in _unzip_iter(filename, root, verbose):
         if isinstance(message, ErrorMessage):
-            raise Exception, message
+            raise Exception(message)
 
 def _unzip_iter(filename, root, verbose=True):
     if verbose:
@@ -1972,16 +1987,21 @@ def _unzip_iter(filename, root, verbose=True):
         sys.stdout.flush()
 
     try: zf = zipfile.ZipFile(filename)
-    except zipfile.error, e:
+    except zipfile.error as e:
         yield ErrorMessage(filename, 'Error with downloaded zip file')
         return
-    except Exception, e:
+    except Exception as e:
         yield ErrorMessage(filename, e)
         return
 
     # Get lists of directories & files
     namelist = zf.namelist()
-    dirlist = [x for x in namelist if x.endswith('/')]
+    dirlist = set()
+    for x in namelist:
+        if x.endswith('/'):
+            dirlist.add(x)
+        else:
+            dirlist.add(x.rsplit('/',1)[0] + '/')
     filelist = [x for x in namelist if not x.endswith('/')]
 
     # Create the target directory if it doesn't exist
@@ -1999,18 +2019,20 @@ def _unzip_iter(filename, root, verbose=True):
     # Extract files.
     for i, filename in enumerate(filelist):
         filepath = os.path.join(root, *filename.split('/'))
-        out = open(filepath, 'wb')
-        try: contents = zf.read(filename)
-        except Exception, e:
-            yield ErrorMessage(filename, e)
-            return
-        out.write(contents)
-        out.close()
+
+        with open(filepath, 'wb') as outfile:
+            try:
+                contents = zf.read(filename)
+            except Exception as e:
+                yield ErrorMessage(filename, e)
+                return
+            outfile.write(contents)
+
         if verbose and (i*10/len(filelist) > (i-1)*10/len(filelist)):
             sys.stdout.write('.')
             sys.stdout.flush()
     if verbose:
-        print
+        print()
 
 ######################################################################
 # Index Builder
@@ -2165,11 +2187,11 @@ def _find_packages(root):
                 xmlfilename = os.path.join(dirname, filename)
                 zipfilename = xmlfilename[:-4]+'.zip'
                 try: zf = zipfile.ZipFile(zipfilename)
-                except Exception, e:
+                except Exception as e:
                     raise ValueError('Error reading file %r!\n%s' %
                                      (zipfilename, e))
                 try: pkg_xml = ElementTree.parse(xmlfilename).getroot()
-                except Exception, e:
+                except Exception as e:
                     raise ValueError('Error reading file %r!\n%s' %
                                      (xmlfilename, e))
 
@@ -2200,32 +2222,43 @@ def _find_packages(root):
 # Aliases
 _downloader = Downloader()
 download = _downloader.download
-def download_shell(): DownloaderShell(_downloader).run()
-def download_gui(): DownloaderGUI(_downloader).mainloop()
-def update(): _downloader.update()
+
+def download_shell():
+    DownloaderShell(_downloader).run()
+
+def download_gui():
+    DownloaderGUI(_downloader).mainloop()
+
+def update():
+    _downloader.update()
 
 if __name__ == '__main__':
     from optparse import OptionParser
     parser = OptionParser()
     parser.add_option("-d", "--dir", dest="dir",
-                      help="download package to directory DIR", metavar="DIR")
+        help="download package to directory DIR", metavar="DIR")
     parser.add_option("-q", "--quiet", dest="quiet", action="store_true",
-                      default=False, help="work quietly")
+        default=False, help="work quietly")
     parser.add_option("-f", "--force", dest="force", action="store_true",
-                      default=False, help="download even if already installed")
+        default=False, help="download even if already installed")
     parser.add_option("-e", "--exit-on-error", dest="halt_on_error", action="store_true",
-                      default=False, help="exit if an error occurs")
+        default=False, help="exit if an error occurs")
+    parser.add_option("-u", "--url", dest="server_index_url",
+        default=None, help="download server index url")
 
     (options, args) = parser.parse_args()
 
+    downloader = Downloader(server_index_url = options.server_index_url)
+
     if args:
         for pkg_id in args:
-            rv = download(info_or_id=pkg_id, download_dir=options.dir,
-                          quiet=options.quiet, force=options.force,
-                          halt_on_error=options.halt_on_error)
+            rv = downloader.download(info_or_id=pkg_id, download_dir=options.dir,
+                quiet=options.quiet, force=options.force,
+                halt_on_error=options.halt_on_error)
             if rv==False and options.halt_on_error:
                 break
     else:
-        download(download_dir=options.dir,
-                 quiet=options.quiet, force=options.force,
-                 halt_on_error=options.halt_on_error)
+        downloader.download(download_dir=options.dir,
+            quiet=options.quiet, force=options.force,
+            halt_on_error=options.halt_on_error)
+
