@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
 # Natural Language Toolkit: Context Free Grammars
 #
-# Copyright (C) 2001-2012 NLTK Project
-# Author: Steven Bird <sb@csse.unimelb.edu.au>
-#         Edward Loper <edloper@seas.upenn.edu>
+# Copyright (C) 2001-2014 NLTK Project
+# Author: Steven Bird <stevenbird1@gmail.com>
+#         Edward Loper <edloper@gmail.com>
 #         Jason Narad <jason.narad@gmail.com>
 #         Peter Ljunglöf <peter.ljunglof@heatherleaf.se>
-# URL: <http://www.nltk.org/>
+# URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
 #
 
@@ -22,8 +22,8 @@ sentences.  In this context, the leaves of a parse tree are word
 tokens; and the node values are phrasal categories, such as ``NP``
 and ``VP``.
 
-The ``ContextFreeGrammar`` class is used to encode context free grammars.  Each
-``ContextFreeGrammar`` consists of a start symbol and a set of productions.
+The ``CFG`` class is used to encode context free grammars.  Each
+``CFG`` consists of a start symbol and a set of productions.
 The "start symbol" specifies the root node value for parse trees.  For example,
 the start symbol for syntactic parsing is usually ``S``.  Start
 symbols are encoded using the ``Nonterminal`` class, which is discussed
@@ -48,8 +48,8 @@ corresponding child may be a ``Token`` with the with that type.
 The ``Nonterminal`` class is used to distinguish node values from leaf
 values.  This prevents the grammar from accidentally using a leaf
 value (such as the English word "A") as the node of a subtree.  Within
-a ``ContextFreeGrammar``, all node values are wrapped in the ``Nonterminal`` class.
-Note, however, that the trees that are specified by the grammar do
+a ``CFG``, all node values are wrapped in the ``Nonterminal``
+class. Note, however, that the trees that are specified by the grammar do
 *not* include these ``Nonterminal`` wrappers.
 
 Grammars can also be given a more procedural interpretation.  According to
@@ -68,18 +68,24 @@ The operation of replacing the left hand side (*lhs*) of a production
 with the right hand side (*rhs*) in a tree (*tree*) is known as
 "expanding" *lhs* to *rhs* in *tree*.
 """
+from __future__ import print_function, unicode_literals
 
 import re
 
 from nltk.util import transitive_closure, invert_graph
+from nltk.compat import (string_types, total_ordering, text_type,
+                         python_2_unicode_compatible, unicode_repr)
+from nltk.internals import raise_unorderable_types
 
 from nltk.probability import ImmutableProbabilisticMixIn
-from nltk.featstruct import FeatStruct, FeatDict, FeatStructParser, SLASH, TYPE
+from nltk.featstruct import FeatStruct, FeatDict, FeatStructReader, SLASH, TYPE
 
 #################################################################
 # Nonterminal
 #################################################################
 
+@total_ordering
+@python_2_unicode_compatible
 class Nonterminal(object):
     """
     A non-terminal symbol for a context free grammar.  ``Nonterminal``
@@ -93,7 +99,7 @@ class Nonterminal(object):
     hashable.  Two ``Nonterminals`` are considered equal if their
     symbols are equal.
 
-    :see: ``ContextFreeGrammar``, ``Production``
+    :see: ``CFG``, ``Production``
     :type _symbol: any
     :ivar _symbol: The node value corresponding to this
         ``Nonterminal``.  This value must be immutable and hashable.
@@ -121,32 +127,20 @@ class Nonterminal(object):
     def __eq__(self, other):
         """
         Return True if this non-terminal is equal to ``other``.  In
-        particular, return True iff ``other`` is a ``Nonterminal``
+        particular, return True if ``other`` is a ``Nonterminal``
         and this non-terminal's symbol is equal to ``other`` 's symbol.
 
         :rtype: bool
         """
-        try:
-            return ((self._symbol == other._symbol) \
-                    and isinstance(other, self.__class__))
-        except AttributeError:
-            return False
+        return type(self) == type(other) and self._symbol == other._symbol
 
     def __ne__(self, other):
-        """
-        Return True if this non-terminal is not equal to ``other``.  In
-        particular, return true iff ``other`` is not a ``Nonterminal``
-        or this non-terminal's symbol is not equal to ``other`` 's symbol.
+        return not self == other
 
-        :rtype: bool
-        """
-        return not (self==other)
-
-    def __cmp__(self, other):
-        try:
-            return cmp(self._symbol, other._symbol)
-        except:
-            return -1
+    def __lt__(self, other):
+        if not isinstance(other, Nonterminal):
+            raise_unorderable_types("<", self, other)
+        return self._symbol < other._symbol
 
     def __hash__(self):
         return self._hash
@@ -157,10 +151,10 @@ class Nonterminal(object):
 
         :rtype: str
         """
-        if isinstance(self._symbol, basestring):
-            return '%s' % (self._symbol,)
+        if isinstance(self._symbol, string_types):
+            return '%s' % self._symbol
         else:
-            return '%r' % (self._symbol,)
+            return '%s' % unicode_repr(self._symbol)
 
     def __str__(self):
         """
@@ -168,10 +162,10 @@ class Nonterminal(object):
 
         :rtype: str
         """
-        if isinstance(self._symbol, basestring):
-            return '%s' % (self._symbol,)
+        if isinstance(self._symbol, string_types):
+            return '%s' % self._symbol
         else:
-            return '%r' % (self._symbol,)
+            return '%s' % unicode_repr(self._symbol)
 
     def __div__(self, rhs):
         """
@@ -237,6 +231,8 @@ def is_terminal(item):
 # Productions
 #################################################################
 
+@total_ordering
+@python_2_unicode_compatible
 class Production(object):
     """
     A grammar production.  Each production maps a single symbol
@@ -248,7 +244,7 @@ class Production(object):
     not a ``Nonterminal``.  Typically, terminals are strings
     representing words, such as ``"dog"`` or ``"under"``.
 
-    :see: ``ContextFreeGrammar``
+    :see: ``CFG``
     :see: ``DependencyGrammar``
     :see: ``Nonterminal``
     :type _lhs: Nonterminal
@@ -266,7 +262,7 @@ class Production(object):
         :param rhs: The right-hand side of the new ``Production``.
         :type rhs: sequence(Nonterminal and terminal)
         """
-        if isinstance(rhs, (str, unicode)):
+        if isinstance(rhs, string_types):
             raise TypeError('production right hand side should be a list, '
                             'not a string')
         self._lhs = lhs
@@ -303,7 +299,7 @@ class Production(object):
 
         :rtype: bool
         """
-        return all([is_nonterminal(n) for n in self._rhs])
+        return all(is_nonterminal(n) for n in self._rhs)
 
     def is_lexical(self):
         """
@@ -319,10 +315,9 @@ class Production(object):
 
         :rtype: str
         """
-        str = '%r ->' % (self._lhs,)
-        for elt in self._rhs:
-            str += ' %r' % (elt,)
-        return str
+        result = '%s -> ' % unicode_repr(self._lhs)
+        result += " ".join(unicode_repr(el) for el in self._rhs)
+        return result
 
     def __repr__(self):
         """
@@ -338,16 +333,17 @@ class Production(object):
 
         :rtype: bool
         """
-        return (isinstance(other, self.__class__) and
+        return (type(self) == type(other) and
                 self._lhs == other._lhs and
                 self._rhs == other._rhs)
 
     def __ne__(self, other):
-        return not (self == other)
+        return not self == other
 
-    def __cmp__(self, other):
-        if not isinstance(other, self.__class__): return -1
-        return cmp((self._lhs, self._rhs), (other._lhs, other._rhs))
+    def __lt__(self, other):
+        if not isinstance(other, Production):
+            raise_unorderable_types("<", self, other)
+        return (self._lhs, self._rhs) < (other._lhs, other._rhs)
 
     def __hash__(self):
         """
@@ -358,6 +354,7 @@ class Production(object):
         return self._hash
 
 
+@python_2_unicode_compatible
 class DependencyProduction(Production):
     """
     A dependency grammar production.  Each production maps a single
@@ -369,47 +366,49 @@ class DependencyProduction(Production):
 
         :rtype: str
         """
-        str = '\'%s\' ->' % (self._lhs,)
+        result = '\'%s\' ->' % (self._lhs,)
         for elt in self._rhs:
-                str += ' \'%s\'' % (elt,)
-        return str
+            result += ' \'%s\'' % (elt,)
+        return result
 
 
-class WeightedProduction(Production, ImmutableProbabilisticMixIn):
+@python_2_unicode_compatible
+class ProbabilisticProduction(Production, ImmutableProbabilisticMixIn):
     """
     A probabilistic context free grammar production.
-    A PCFG ``WeightedProduction`` is essentially just a ``Production`` that
+    A PCFG ``ProbabilisticProduction`` is essentially just a ``Production`` that
     has an associated probability, which represents how likely it is that
     this production will be used.  In particular, the probability of a
-    ``WeightedProduction`` records the likelihood that its right-hand side is
+    ``ProbabilisticProduction`` records the likelihood that its right-hand side is
     the correct instantiation for any given occurrence of its left-hand side.
 
     :see: ``Production``
     """
     def __init__(self, lhs, rhs, **prob):
         """
-        Construct a new ``WeightedProduction``.
+        Construct a new ``ProbabilisticProduction``.
 
-        :param lhs: The left-hand side of the new ``WeightedProduction``.
+        :param lhs: The left-hand side of the new ``ProbabilisticProduction``.
         :type lhs: Nonterminal
-        :param rhs: The right-hand side of the new ``WeightedProduction``.
+        :param rhs: The right-hand side of the new ``ProbabilisticProduction``.
         :type rhs: sequence(Nonterminal and terminal)
-        :param prob: Probability parameters of the new ``WeightedProduction``.
+        :param prob: Probability parameters of the new ``ProbabilisticProduction``.
         """
         ImmutableProbabilisticMixIn.__init__(self, **prob)
         Production.__init__(self, lhs, rhs)
 
     def __str__(self):
-        return Production.__str__(self) + ' [%s]' % self.prob()
+        return Production.__unicode__(self) + \
+            (' [1.0]' if (self.prob() == 1.0) else ' [%g]' % self.prob())
 
     def __eq__(self, other):
-        return (isinstance(other, self.__class__) and
+        return (type(self) == type(other) and
                 self._lhs == other._lhs and
                 self._rhs == other._rhs and
                 self.prob() == other.prob())
 
     def __ne__(self, other):
-        return not (self == other)
+        return not self == other
 
     def __hash__(self):
         return hash((self._lhs, self._rhs, self.prob()))
@@ -418,7 +417,8 @@ class WeightedProduction(Production, ImmutableProbabilisticMixIn):
 # Grammars
 #################################################################
 
-class ContextFreeGrammar(object):
+@python_2_unicode_compatible
+class CFG(object):
     """
     A context-free grammar.  A grammar consists of a start state and
     a set of productions.  The set of terminals and nonterminals is
@@ -440,6 +440,10 @@ class ContextFreeGrammar(object):
             leftcorner relation. In that case, some optimized chart parsers won't work.
         :type calculate_leftcorners: bool
         """
+        if not is_nonterminal(start):
+            raise TypeError("start should be a Nonterminal object,"
+                            " not a %s" % type(start).__name__)
+
         self._start = start
         self._productions = productions
         self._categories = set(prod.lhs() for prod in productions)
@@ -498,11 +502,22 @@ class ContextFreeGrammar(object):
             return
 
         self._leftcorner_words = {}
-        for cat, lefts in self._leftcorners.iteritems():
+        for cat in self._leftcorners:
+            lefts = self._leftcorners[cat]
             lc = self._leftcorner_words[cat] = set()
             for left in lefts:
                 lc.update(self._immediate_leftcorner_words.get(left, set()))
 
+    @classmethod
+    def fromstring(cls, input, encoding=None):
+        """
+        Return the ``CFG`` corresponding to the input string(s).
+
+        :param input: a grammar, either in the form of a string or as a list of strings.
+        """
+        start, productions = read_grammar(input, standard_nonterm_parser,
+                                          encoding=encoding)
+        return CFG(start, productions)
 
     def start(self):
         """
@@ -586,8 +601,8 @@ class ContextFreeGrammar(object):
         elif self._leftcorner_words:
             return left in self._leftcorner_words.get(cat, set())
         else:
-            return any([left in _immediate_leftcorner_words.get(parent, set())
-                        for parent in self.leftcorners(cat)])
+            return any(left in self._immediate_leftcorner_words.get(parent, set())
+                       for parent in self.leftcorners(cat))
 
     def leftcorner_parents(self, cat):
         """
@@ -620,13 +635,13 @@ class ContextFreeGrammar(object):
         Pre-calculate of which form(s) the grammar is.
         """
         prods = self._productions
-        self._is_lexical = all([p.is_lexical() for p in prods])
-        self._is_nonlexical = all([p.is_nonlexical() for p in prods
-                                  if len(p) != 1])
+        self._is_lexical = all(p.is_lexical() for p in prods)
+        self._is_nonlexical = all(p.is_nonlexical() for p in prods
+                                  if len(p) != 1)
         self._min_len = min(len(p) for p in prods)
         self._max_len = max(len(p) for p in prods)
-        self._all_unary_are_lexical = all([p.is_lexical() for p in prods
-                                          if len(p) == 1])
+        self._all_unary_are_lexical = all(p.is_lexical() for p in prods
+                                          if len(p) == 1)
 
     def is_lexical(self):
         """
@@ -691,17 +706,17 @@ class ContextFreeGrammar(object):
         return '<Grammar with %d productions>' % len(self._productions)
 
     def __str__(self):
-        str = 'Grammar with %d productions' % len(self._productions)
-        str += ' (start state = %r)' % self._start
+        result = 'Grammar with %d productions' % len(self._productions)
+        result += ' (start state = %r)' % self._start
         for production in self._productions:
-            str += '\n    %s' % production
-        return str
+            result += '\n    %s' % production
+        return result
 
 
-class FeatureGrammar(ContextFreeGrammar):
+class FeatureGrammar(CFG):
     """
     A feature-based grammar.  This is equivalent to a
-    ``ContextFreeGrammar`` whose nonterminals are all
+    ``CFG`` whose nonterminals are all
     ``FeatStructNonterminal``.
 
     A grammar consists of a start state and a set of
@@ -718,7 +733,7 @@ class FeatureGrammar(ContextFreeGrammar):
         :param productions: The list of productions that defines the grammar
         :type productions: list(Production)
         """
-        ContextFreeGrammar.__init__(self, start, productions)
+        CFG.__init__(self, start, productions)
 
     # The difference with CFG is that the productions are
     # indexed on the TYPE feature of the nonterminals.
@@ -753,6 +768,35 @@ class FeatureGrammar(ContextFreeGrammar):
                 if is_terminal(token):
                     self._lexical_index.setdefault(token, set()).add(prod)
 
+    @classmethod
+    def fromstring(cls, input, features=None, logic_parser=None, fstruct_reader=None,
+               encoding=None):
+        """
+        Return a feature structure based ``FeatureGrammar``.
+
+        :param input: a grammar, either in the form of a string or else
+        as a list of strings.
+        :param features: a tuple of features (default: SLASH, TYPE)
+        :param logic_parser: a parser for lambda-expressions,
+        by default, ``LogicParser()``
+        :param fstruct_reader: a feature structure parser
+        (only if features and logic_parser is None)
+        """
+        if features is None:
+            features = (SLASH, TYPE)
+
+        if fstruct_reader is None:
+            fstruct_reader = FeatStructReader(features, FeatStructNonterminal,
+                                              logic_parser=logic_parser)
+        elif logic_parser is not None:
+            raise Exception('\'logic_parser\' and \'fstruct_reader\' must '
+                            'not both be set')
+
+        start, productions = read_grammar(input, fstruct_reader.read_partial,
+                                          encoding=encoding)
+        return FeatureGrammar(start, productions)
+
+
     def productions(self, lhs=None, rhs=None, empty=False):
         """
         Return the grammar productions, filtered by the left-hand side
@@ -815,6 +859,8 @@ class FeatureGrammar(ContextFreeGrammar):
         else:
             return item
 
+@total_ordering
+@python_2_unicode_compatible
 class FeatureValueType(object):
     """
     A helper class for ``FeatureGrammars``, designed to be different
@@ -824,13 +870,26 @@ class FeatureValueType(object):
     def __init__(self, value):
         self._value = value
         self._hash = hash(value)
+
     def __repr__(self):
-        return '<%s>' % self.value
-    def __cmp__(self, other):
-        return cmp(FeatureValueType, type(other)) or cmp(self._value, other._value)
+        return '<%s>' % self._value
+
+    def __eq__(self, other):
+        return type(self) == type(other) and self._value == other._value
+
+    def __ne__(self, other):
+        return not self == other
+
+    def __lt__(self, other):
+        if not isinstance(other, FeatureValueType):
+            raise_unorderable_types("<", self, other)
+        return self._value < other._value
+
     def __hash__(self):
         return self._hash
 
+
+@python_2_unicode_compatible
 class DependencyGrammar(object):
     """
     A dependency grammar.  A DependencyGrammar consists of a set of
@@ -846,6 +905,19 @@ class DependencyGrammar(object):
         """
         self._productions = productions
 
+    @classmethod
+    def fromstring(cls, input):
+        productions = []
+        for linenum, line in enumerate(input.split('\n')):
+            line = line.strip()
+            if line.startswith('#') or line=='': continue
+            try: productions += _read_dependency_production(line)
+            except ValueError:
+                raise ValueError('Unable to parse line %s: %s' % (linenum, line))
+        if len(productions) == 0:
+            raise ValueError('No productions found!')
+        return DependencyGrammar(productions)
+
     def contains(self, head, mod):
         """
         :param head: A head word.
@@ -910,7 +982,8 @@ class DependencyGrammar(object):
         return 'Dependency grammar with %d productions' % len(self._productions)
 
 
-class StatisticalDependencyGrammar(object):
+@python_2_unicode_compatible
+class ProbabilisticDependencyGrammar(object):
     """
 
     """
@@ -939,7 +1012,7 @@ class StatisticalDependencyGrammar(object):
 
     def __str__(self):
         """
-        Return a verbose string representation of the ``StatisticalDependencyGrammar``
+        Return a verbose string representation of the ``ProbabilisticDependencyGrammar``
 
         :rtype: str
         """
@@ -956,21 +1029,21 @@ class StatisticalDependencyGrammar(object):
 
     def __repr__(self):
         """
-        Return a concise string representation of the ``StatisticalDependencyGrammar``
+        Return a concise string representation of the ``ProbabilisticDependencyGrammar``
         """
         return 'Statistical Dependency grammar with %d productions' % len(self._productions)
 
 
-class WeightedGrammar(ContextFreeGrammar):
+class PCFG(CFG):
     """
-    A probabilistic context-free grammar.  A Weighted Grammar consists
-    of a start state and a set of weighted productions.  The set of
+    A probabilistic context-free grammar.  A PCFG consists of a
+    start state and a set of productions with probabilities.  The set of
     terminals and nonterminals is implicitly specified by the productions.
 
-    PCFG productions should be ``WeightedProductions``.
-    ``WeightedGrammars`` impose the constraint that the set of
-    productions with any given left-hand-side must have probabilities
-    that sum to 1.
+    PCFG productions use the ``ProbabilisticProduction`` class.
+    ``PCFGs`` impose the constraint that the set of productions with
+    any given left-hand-side must have probabilities that sum to 1
+    (allowing for a small margin of error).
 
     If you need efficient key-based access to productions, you can use
     a subclass to implement it.
@@ -985,7 +1058,7 @@ class WeightedGrammar(ContextFreeGrammar):
     def __init__(self, start, productions, calculate_leftcorners=True):
         """
         Create a new context-free grammar, from the given start state
-        and set of ``WeightedProductions``.
+        and set of ``ProbabilisticProductions``.
 
         :param start: The start symbol
         :type start: Nonterminal
@@ -998,7 +1071,7 @@ class WeightedGrammar(ContextFreeGrammar):
             leftcorner relation. In that case, some optimized chart parsers won't work.
         :type calculate_leftcorners: bool
         """
-        ContextFreeGrammar.__init__(self, start, productions, calculate_leftcorners)
+        CFG.__init__(self, start, productions, calculate_leftcorners)
 
         # Make sure that the probabilities sum to one.
         probs = {}
@@ -1006,11 +1079,25 @@ class WeightedGrammar(ContextFreeGrammar):
             probs[production.lhs()] = (probs.get(production.lhs(), 0) +
                                        production.prob())
         for (lhs, p) in probs.items():
-            if not ((1-WeightedGrammar.EPSILON) < p <
-                    (1+WeightedGrammar.EPSILON)):
+            if not ((1-PCFG.EPSILON) < p <
+                    (1+PCFG.EPSILON)):
                 raise ValueError("Productions for %r do not sum to 1" % lhs)
 
 
+    @classmethod
+    def fromstring(cls, input, encoding=None):
+        """
+        Return a probabilistic ``PCFG`` corresponding to the
+        input string(s).
+
+        :param input: a grammar, either in the form of a string or else
+             as a list of strings.
+        """
+        start, productions = read_grammar(input, standard_nonterm_parser,
+                                          probabilistic=True, encoding=encoding)
+        return PCFG(start, productions)
+
+
 #################################################################
 # Inducing Grammars
 #################################################################
@@ -1032,7 +1119,6 @@ def induce_pcfg(start, productions):
     :param productions: The list of productions that defines the grammar
     :type productions: list(Production)
     """
-
     # Production count: the number of times a given production occurs
     pcount = {}
 
@@ -1043,86 +1129,34 @@ def induce_pcfg(start, productions):
         lcount[prod.lhs()] = lcount.get(prod.lhs(), 0) + 1
         pcount[prod]       = pcount.get(prod,       0) + 1
 
-    prods = [WeightedProduction(p.lhs(), p.rhs(),
+    prods = [ProbabilisticProduction(p.lhs(), p.rhs(),
                                 prob=float(pcount[p]) / lcount[p.lhs()])
              for p in pcount]
-    return WeightedGrammar(start, prods)
+    return PCFG(start, prods)
 
 
 #################################################################
-# Parsing Grammars
+# Helper functions for reading productions
 #################################################################
 
-# Parsing CFGs
-
-def parse_cfg_production(input):
+def _read_cfg_production(input):
     """
     Return a list of context-free ``Productions``.
     """
-    return parse_production(input, standard_nonterm_parser)
+    return _read_production(input, standard_nonterm_parser)
 
-def parse_cfg(input):
+def _read_pcfg_production(input):
     """
-    Return the ``ContextFreeGrammar`` corresponding to the input string(s).
-
-    :param input: a grammar, either in the form of a string or
-        as a list of strings.
+    Return a list of PCFG ``ProbabilisticProductions``.
     """
-    start, productions = parse_grammar(input, standard_nonterm_parser)
-    return ContextFreeGrammar(start, productions)
+    return _read_production(input, standard_nonterm_parser, probabilistic=True)
 
-# Parsing Probabilistic CFGs
-
-def parse_pcfg_production(input):
-    """
-    Return a list of PCFG ``WeightedProductions``.
-    """
-    return parse_production(input, standard_nonterm_parser, probabilistic=True)
-
-def parse_pcfg(input):
-    """
-    Return a probabilistic ``WeightedGrammar`` corresponding to the
-    input string(s).
-
-    :param input: a grammar, either in the form of a string or else
-        as a list of strings.
-    """
-    start, productions = parse_grammar(input, standard_nonterm_parser,
-                                       probabilistic=True)
-    return WeightedGrammar(start, productions)
-
-# Parsing Feature-based CFGs
-
-def parse_fcfg_production(input, fstruct_parser):
+def _read_fcfg_production(input, fstruct_reader):
     """
     Return a list of feature-based ``Productions``.
     """
-    return parse_production(input, fstruct_parser)
+    return _read_production(input, fstruct_reader)
 
-def parse_fcfg(input, features=None, logic_parser=None, fstruct_parser=None):
-    """
-    Return a feature structure based ``FeatureGrammar``.
-
-    :param input: a grammar, either in the form of a string or else
-        as a list of strings.
-    :param features: a tuple of features (default: SLASH, TYPE)
-    :param logic_parser: a parser for lambda-expressions,
-        by default, ``LogicParser()``
-    :param fstruct_parser: a feature structure parser
-        (only if features and logic_parser is None)
-    """
-    if features is None:
-        features = (SLASH, TYPE)
-
-    if fstruct_parser is None:
-        fstruct_parser = FeatStructParser(features, FeatStructNonterminal,
-                                          logic_parser=logic_parser)
-    elif logic_parser is not None:
-        raise Exception('\'logic_parser\' and \'fstruct_parser\' must '
-                        'not both be set')
-
-    start, productions = parse_grammar(input, fstruct_parser.partial_parse)
-    return FeatureGrammar(start, productions)
 
 # Parsing generic grammars
 
@@ -1131,7 +1165,7 @@ _PROBABILITY_RE = re.compile(r'( \[ [\d\.]+ \] ) \s*', re.VERBOSE)
 _TERMINAL_RE = re.compile(r'( "[^"]+" | \'[^\']+\' ) \s*', re.VERBOSE)
 _DISJUNCTION_RE = re.compile(r'\| \s*', re.VERBOSE)
 
-def parse_production(line, nonterm_parser, probabilistic=False):
+def _read_production(line, nonterm_parser, probabilistic=False):
     """
     Parse a grammar rule, given as a string, and return
     a list of productions.
@@ -1180,13 +1214,17 @@ def parse_production(line, nonterm_parser, probabilistic=False):
             rhsides[-1].append(nonterm)
 
     if probabilistic:
-        return [WeightedProduction(lhs, rhs, prob=probability)
+        return [ProbabilisticProduction(lhs, rhs, prob=probability)
                 for (rhs, probability) in zip(rhsides, probabilities)]
     else:
         return [Production(lhs, rhs) for rhs in rhsides]
 
 
-def parse_grammar(input, nonterm_parser, probabilistic=False):
+#################################################################
+# Reading Phrase Structure Grammars
+#################################################################
+
+def read_grammar(input, nonterm_parser, probabilistic=False, encoding=None):
     """
     Return a pair consisting of a starting category and a list of
     ``Productions``.
@@ -1198,8 +1236,12 @@ def parse_grammar(input, nonterm_parser, probabilistic=False):
         return a ``(nonterminal, position)`` as result.
     :param probabilistic: are the grammar rules probabilistic?
     :type probabilistic: bool
+    :param encoding: the encoding of the grammar, if it is a binary string
+    :type encoding: str
     """
-    if isinstance(input, basestring):
+    if encoding is not None:
+        input = input.decode(encoding)
+    if isinstance(input, string_types):
         lines = input.split('\n')
     else:
         lines = input
@@ -1225,13 +1267,13 @@ def parse_grammar(input, nonterm_parser, probabilistic=False):
                     raise ValueError('Bad directive')
             else:
                 # expand out the disjunctions on the RHS
-                productions += parse_production(line, nonterm_parser, probabilistic)
-        except ValueError, e:
+                productions += _read_production(line, nonterm_parser, probabilistic)
+        except ValueError as e:
             raise ValueError('Unable to parse line %s: %s\n%s' %
                              (linenum+1, line, e))
 
     if not productions:
-        raise ValueError, 'No productions found!'
+        raise ValueError('No productions found!')
     if not start:
         start = productions[0].lhs()
     return (start, productions)
@@ -1246,10 +1288,10 @@ def standard_nonterm_parser(string, pos):
 
 
 #################################################################
-# Parsing Dependency Grammars
+# Reading Dependency Grammars
 #################################################################
 
-_PARSE_DG_RE = re.compile(r'''^\s*                # leading whitespace
+_READ_DG_RE = re.compile(r'''^\s*                # leading whitespace
                               ('[^']+')\s*        # single-quoted lhs
                               (?:[-=]+>)\s*        # arrow
                               (?:(                 # rhs:
@@ -1262,21 +1304,9 @@ _PARSE_DG_RE = re.compile(r'''^\s*                # leading whitespace
                              re.VERBOSE)
 _SPLIT_DG_RE = re.compile(r'''('[^']'|[-=]+>|"[^"]+"|'[^']+'|\|)''')
 
-def parse_dependency_grammar(s):
-    productions = []
-    for linenum, line in enumerate(s.split('\n')):
-        line = line.strip()
-        if line.startswith('#') or line=='': continue
-        try: productions += parse_dependency_production(line)
-        except ValueError:
-            raise ValueError, 'Unable to parse line %s: %s' % (linenum, line)
-    if len(productions) == 0:
-        raise ValueError, 'No productions found!'
-    return DependencyGrammar(productions)
-
-def parse_dependency_production(s):
-    if not _PARSE_DG_RE.match(s):
-        raise ValueError, 'Bad production string'
+def _read_dependency_production(s):
+    if not _READ_DG_RE.match(s):
+        raise ValueError('Bad production string')
     pieces = _SPLIT_DG_RE.split(s)
     pieces = [p for i,p in enumerate(pieces) if i%2==1]
     lhside = pieces[0].strip('\'\"')
@@ -1295,24 +1325,24 @@ def parse_dependency_production(s):
 
 def cfg_demo():
     """
-    A demonstration showing how ``ContextFreeGrammars`` can be created and used.
+    A demonstration showing how ``CFGs`` can be created and used.
     """
 
-    from nltk import nonterminals, Production, parse_cfg
+    from nltk import nonterminals, Production, CFG
 
     # Create some nonterminals
     S, NP, VP, PP = nonterminals('S, NP, VP, PP')
     N, V, P, Det = nonterminals('N, V, P, Det')
     VP_slash_NP = VP/NP
 
-    print 'Some nonterminals:', [S, NP, VP, PP, N, V, P, Det, VP/NP]
-    print '    S.symbol() =>', `S.symbol()`
-    print
+    print('Some nonterminals:', [S, NP, VP, PP, N, V, P, Det, VP/NP])
+    print('    S.symbol() =>', repr(S.symbol()))
+    print()
 
-    print Production(S, [NP])
+    print(Production(S, [NP]))
 
     # Create some Grammar Productions
-    grammar = parse_cfg("""
+    grammar = CFG.fromstring("""
       S -> NP VP
       PP -> P NP
       NP -> Det N | NP PP
@@ -1323,14 +1353,14 @@ def cfg_demo():
       P -> 'on' | 'in'
     """)
 
-    print 'A Grammar:', `grammar`
-    print '    grammar.start()       =>', `grammar.start()`
-    print '    grammar.productions() =>',
+    print('A Grammar:', repr(grammar))
+    print('    grammar.start()       =>', repr(grammar.start()))
+    print('    grammar.productions() =>', end=' ')
     # Use string.replace(...) is to line-wrap the output.
-    print `grammar.productions()`.replace(',', ',\n'+' '*25)
-    print
+    print(repr(grammar.productions()).replace(',', ',\n'+' '*25))
+    print()
 
-toy_pcfg1 = parse_pcfg("""
+toy_pcfg1 = PCFG.fromstring("""
     S -> NP VP [1.0]
     NP -> Det N [0.5] | NP PP [0.25] | 'John' [0.1] | 'I' [0.15]
     Det -> 'the' [0.8] | 'my' [0.2]
@@ -1341,7 +1371,7 @@ toy_pcfg1 = parse_pcfg("""
     P -> 'with' [0.61] | 'under' [0.39]
     """)
 
-toy_pcfg2 = parse_pcfg("""
+toy_pcfg2 = PCFG.fromstring("""
     S    -> NP VP         [1.0]
     VP   -> V NP          [.59]
     VP   -> V             [.40]
@@ -1369,7 +1399,7 @@ toy_pcfg2 = parse_pcfg("""
 
 def pcfg_demo():
     """
-    A demonstration showing how a ``WeightedGrammar`` can be created and used.
+    A demonstration showing how a ``PCFG`` can be created and used.
     """
 
     from nltk.corpus import treebank
@@ -1380,22 +1410,22 @@ def pcfg_demo():
     pcfg_prods = toy_pcfg1.productions()
 
     pcfg_prod = pcfg_prods[2]
-    print 'A PCFG production:', `pcfg_prod`
-    print '    pcfg_prod.lhs()  =>', `pcfg_prod.lhs()`
-    print '    pcfg_prod.rhs()  =>', `pcfg_prod.rhs()`
-    print '    pcfg_prod.prob() =>', `pcfg_prod.prob()`
-    print
+    print('A PCFG production:', repr(pcfg_prod))
+    print('    pcfg_prod.lhs()  =>', repr(pcfg_prod.lhs()))
+    print('    pcfg_prod.rhs()  =>', repr(pcfg_prod.rhs()))
+    print('    pcfg_prod.prob() =>', repr(pcfg_prod.prob()))
+    print()
 
     grammar = toy_pcfg2
-    print 'A PCFG grammar:', `grammar`
-    print '    grammar.start()       =>', `grammar.start()`
-    print '    grammar.productions() =>',
-    # Use string.replace(...) is to line-wrap the output.
-    print `grammar.productions()`.replace(',', ',\n'+' '*26)
-    print
+    print('A PCFG grammar:', repr(grammar))
+    print('    grammar.start()       =>', repr(grammar.start()))
+    print('    grammar.productions() =>', end=' ')
+    # Use .replace(...) is to line-wrap the output.
+    print(repr(grammar.productions()).replace(',', ',\n'+' '*26))
+    print()
 
     # extract productions from three trees and induce the PCFG
-    print "Induce PCFG grammar from treebank data:"
+    print("Induce PCFG grammar from treebank data:")
 
     productions = []
     item = treebank._fileids[0]
@@ -1408,10 +1438,10 @@ def pcfg_demo():
 
     S = Nonterminal('S')
     grammar = induce_pcfg(S, productions)
-    print grammar
-    print
+    print(grammar)
+    print()
 
-    print "Parse sentence using induced grammar:"
+    print("Parse sentence using induced grammar:")
 
     parser = pchart.InsideChartParser(grammar)
     parser.trace(3)
@@ -1420,27 +1450,27 @@ def pcfg_demo():
     #sent = treebank.tokenized('wsj_0001.mrg')[0]
 
     sent = treebank.parsed_sents(item)[0].leaves()
-    print sent
-    for parse in parser.nbest_parse(sent):
-        print parse
+    print(sent)
+    for parse in parser.parse(sent):
+        print(parse)
 
 def fcfg_demo():
     import nltk.data
     g = nltk.data.load('grammars/book_grammars/feat0.fcfg')
-    print g
-    print
+    print(g)
+    print()
 
 def dg_demo():
     """
     A demonstration showing the creation and inspection of a
     ``DependencyGrammar``.
     """
-    grammar = parse_dependency_grammar("""
+    grammar = DependencyGrammar.fromstring("""
     'scratch' -> 'cats' | 'walls'
     'walls' -> 'the'
     'cats' -> 'the'
     """)
-    print grammar
+    print(grammar)
 
 def sdg_demo():
     """
@@ -1465,7 +1495,7 @@ def sdg_demo():
     13  .                 .                 Punc  Punc  punt                             12  punct   _  _
     """)
     tree = dg.tree()
-    print tree.pprint()
+    print(tree.pprint())
 
 def demo():
     cfg_demo()
@@ -1478,14 +1508,9 @@ if __name__ == '__main__':
     demo()
 
 __all__ = ['Nonterminal', 'nonterminals',
-           'Production', 'DependencyProduction', 'WeightedProduction',
-           'ContextFreeGrammar', 'WeightedGrammar', 'DependencyGrammar',
-           'StatisticalDependencyGrammar',
-           'induce_pcfg', 'parse_cfg', 'parse_cfg_production',
-           'parse_pcfg', 'parse_pcfg_production',
-           'parse_fcfg', 'parse_fcfg_production',
-           'parse_grammar', 'parse_production',
-           'parse_dependency_grammar', 'parse_dependency_production',
-           'demo', 'cfg_demo', 'pcfg_demo', 'dg_demo', 'sdg_demo',
-           'toy_pcfg1', 'toy_pcfg2']
+           'CFG', 'Production',
+	   'PCFG', 'ProbabilisticProduction',
+	   'DependencyGrammar', 'DependencyProduction',
+           'ProbabilisticDependencyGrammar',
+	   'induce_pcfg', 'read_grammar']
 
