@@ -1,16 +1,18 @@
 # Natural Language Toolkit: Stemmers
 #
-# Copyright (C) 2001-2012 NLTK Project
+# Copyright (C) 2001-2014 NLTK Project
 # Author: Trevor Cohn <tacohn@cs.mu.oz.au>
-#         Edward Loper <edloper@gradient.cis.upenn.edu>
-#         Steven Bird <sb@csse.unimelb.edu.au>
-# URL: <http://www.nltk.org/>
+#         Edward Loper <edloper@gmail.com>
+#         Steven Bird <stevenbird1@gmail.com>
+# URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
-
+from __future__ import unicode_literals
 import re
 
-from api import StemmerI
+from nltk.stem.api import StemmerI
+from nltk.compat import python_2_unicode_compatible
 
+@python_2_unicode_compatible
 class RegexpStemmer(StemmerI):
     """
     A stemmer that uses regular expressions to identify morphological
@@ -18,7 +20,7 @@ class RegexpStemmer(StemmerI):
     be removed.
 
         >>> from nltk.stem import RegexpStemmer
-        >>> st = RegexpStemmer('ing$|s$|e$', min=4)
+        >>> st = RegexpStemmer('ing$|s$|e$|able$', min=4)
         >>> st.stem('cars')
         'car'
         >>> st.stem('mass')
@@ -29,6 +31,8 @@ class RegexpStemmer(StemmerI):
         'bee'
         >>> st.stem('compute')
         'comput'
+        >>> st.stem('advisable')
+        'advis'
 
     :type regexp: str or regexp
     :param regexp: The regular expression that should be used to
