@@ -1,33 +1,45 @@
 # -*- coding: utf-8 -*-
 # Natural Language Toolkit: Tokenizers
 #
-# Copyright (C) 2001-2012 NLTK Project
-# Author: Edward Loper <edloper@gradient.cis.upenn.edu>
-#         Steven Bird <sb@csse.unimelb.edu.au> (minor additions)
-# URL: <http://www.nltk.org/>
+# Copyright (C) 2001-2014 NLTK Project
+# Author: Edward Loper <edloper@gmail.com>
+#         Steven Bird <stevenbird1@gmail.com> (minor additions)
+# URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
 
 r"""
 NLTK Tokenizer Package
 
 Tokenizers divide strings into lists of substrings.  For example,
-tokenizers can be used to find the list of sentences or words in a
-string.
+tokenizers can be used to find the words and punctuation in a string:
 
-    >>> from nltk.tokenize import word_tokenize, wordpunct_tokenize, sent_tokenize
+    >>> from nltk.tokenize import word_tokenize
     >>> s = '''Good muffins cost $3.88\nin New York.  Please buy me
     ... two of them.\n\nThanks.'''
+    >>> word_tokenize(s)
+    ['Good', 'muffins', 'cost', '$', '3.88', 'in', 'New', 'York', '.',
+    'Please', 'buy', 'me', 'two', 'of', 'them', '.', 'Thanks', '.']
+
+This particular tokenizer requires the Punkt sentence tokenization
+models to be installed. NLTK also provides a simpler,
+regular-expression based tokenizer, which splits text on whitespace
+and punctuation:
+
+    >>> from nltk.tokenize import wordpunct_tokenize
     >>> wordpunct_tokenize(s)
     ['Good', 'muffins', 'cost', '$', '3', '.', '88', 'in', 'New', 'York', '.',
     'Please', 'buy', 'me', 'two', 'of', 'them', '.', 'Thanks', '.']
+
+We can also operate at the level of sentences, using the sentence
+tokenizer directly as follows:
+
+    >>> from nltk.tokenize import sent_tokenize, word_tokenize
     >>> sent_tokenize(s)
     ['Good muffins cost $3.88\nin New York.', 'Please buy me\ntwo of them.', 'Thanks.']
     >>> [word_tokenize(t) for t in sent_tokenize(s)]
     [['Good', 'muffins', 'cost', '$', '3.88', 'in', 'New', 'York', '.'],
     ['Please', 'buy', 'me', 'two', 'of', 'them', '.'], ['Thanks', '.']]
 
-Caution: only use ``word_tokenize()`` on individual sentences.
-
 Caution: when tokenizing a Unicode string, make sure you are not
 using an encoded version of the string (it may be necessary to
 decode it first, e.g. with ``s.decode("utf8")``.
@@ -57,13 +69,7 @@ from nltk.tokenize.regexp   import (RegexpTokenizer, WhitespaceTokenizer,
 from nltk.tokenize.punkt    import PunktSentenceTokenizer, PunktWordTokenizer
 from nltk.tokenize.sexpr    import SExprTokenizer, sexpr_tokenize
 from nltk.tokenize.treebank import TreebankWordTokenizer
-
-try:
-    import numpy
-except ImportError:
-    pass
-else:
-    from nltk.tokenize.texttiling import TextTilingTokenizer
+from nltk.tokenize.texttiling import TextTilingTokenizer
 
 # Standard sentence tokenizer.
 def sent_tokenize(text):
@@ -76,16 +82,16 @@ def sent_tokenize(text):
     return tokenizer.tokenize(text)
 
 # Standard word tokenizer.
-_word_tokenize = TreebankWordTokenizer().tokenize
+_treebank_word_tokenize = TreebankWordTokenizer().tokenize
 def word_tokenize(text):
     """
     Return a tokenized copy of *text*,
     using NLTK's recommended word tokenizer
-    (currently :class:`.TreebankWordTokenizer`).
-    This tokenizer is designed to work on a sentence at a time.
+    (currently :class:`.TreebankWordTokenizer`
+    along with :class:`.PunktSentenceTokenizer`).
     """
-    return _word_tokenize(text)
-
+    return [token for sent in sent_tokenize(text)
+            for token in _treebank_word_tokenize(sent)]
 
 if __name__ == "__main__":
     import doctest
